@@ -37,39 +37,41 @@
  *      limitations under the License.
  */
 
-#include <QFile>
-#include <QCryptographicHash>
 #include <Version.h>
-#include <QDir>
-#include <QJsonDocument>
-#include <QJsonArray>
+#include <qlogging.h>
+#include <QCryptographicHash>
 #include <QDebug>
+#include <QDir>
+#include <QFile>
+#include <QJsonArray>
+#include <QJsonDocument>
 #include <QSaveFile>
-#include <QUuid>
 #include <QTimer>
-
-#include "Exception.h"
-#include "minecraft/OneSixVersionFormat.h"
-#include "FileSystem.h"
-#include "minecraft/MinecraftInstance.h"
-#include "minecraft/ProfileUtils.h"
-#include "Json.h"
-
-#include "PackProfile.h"
-#include "PackProfile_p.h"
-#include "ComponentUpdateTask.h"
+#include <QUuid>
+#include <algorithm>
+#include <utility>
 
 #include "Application.h"
-#include "modplatform/ResourceAPI.h"
+#include "Exception.h"
+#include "FileSystem.h"
+#include "Json.h"
+#include "meta/Index.h"
+#include "meta/JsonFormat.h"
+#include "minecraft/Component.h"
+#include "minecraft/MinecraftInstance.h"
+#include "minecraft/OneSixVersionFormat.h"
+#include "minecraft/ProfileUtils.h"
 
-static const QMap<QString, ResourceAPI::ModLoaderType> modloaderMapping{
-    {"net.minecraftforge", ResourceAPI::Forge},
-    {"net.fabricmc.fabric-loader", ResourceAPI::Fabric},
-    {"org.quiltmc.quilt-loader", ResourceAPI::Quilt}
-};
+#include "ComponentUpdateTask.h"
+#include "PackProfile.h"
+#include "PackProfile_p.h"
+#include "modplatform/ModIndex.h"
 
-PackProfile::PackProfile(MinecraftInstance * instance)
-    : QAbstractListModel()
+#include "minecraft/Logging.h"
+
+#include "ui/dialogs/CustomMessageBox.h"
+
+PackProfile::PackProfile(MinecraftInstance* instance) : QAbstractListModel()
 {
     d.reset(new PackProfileData);
     d->m_instance = instance;
@@ -94,42 +96,35 @@ static QJsonObject componentToJsonV1(ComponentPtr component)
     QJsonObject obj;
     // critical
     obj.insert("uid", component->m_uid);
-    if(!component->m_version.isEmpty())
-    {
+    if (!component->m_version.isEmpty()) {
         obj.insert("version", component->m_version);
     }
-    if(component->m_dependencyOnly)
-    {
+    if (component->m_dependencyOnly) {
         obj.insert("dependencyOnly", true);
     }
-    if(component->m_important)
-    {
+    if (component->m_important) {
         obj.insert("important", true);
     }
-    if(component->m_disabled)
-    {
+    if (component->m_disabled) {
         obj.insert("disabled", true);
     }
 
     // cached
-    if(!component->m_cachedVersion.isEmpty())
-    {
+    if (!component->m_cachedVersion.isEmpty()) {
         obj.insert("cachedVersion", component->m_cachedVersion);
     }
-    if(!component->m_cachedName.isEmpty())
-    {
+    if (!component->m_cachedName.isEmpty()) {
         obj.insert("cachedName", component->m_cachedName);
     }
     Meta::serializeRequires(obj, &component->m_cachedRequires, "cachedRequires");
     Meta::serializeRequires(obj, &component->m_cachedConflicts, "cachedConflicts");
-    if(component->m_cachedVolatile)
-    {
+    if (component->m_cachedVolatile) {
         obj.insert("cachedVolatile", true);
     }
     return obj;
 }
 
-static ComponentPtr componentFromJsonV1(PackProfile * parent, const QString & componentJsonPattern, const QJsonObject &obj)
+static ComponentPtr componentFromJsonV1(PackProfile* parent, const QString& componentJsonPattern, const QJsonObject& obj)
 {
     // critical
     auto uid = Json::requireString(obj.value("uid"));
@@ -152,86 +147,72 @@ static ComponentPtr componentFromJsonV1(PackProfile * parent, const QString & co
 }
 
 // Save the given component container data to a file
-static bool savePackProfile(const QString & filename, const ComponentContainer & container)
+static bool savePackProfile(const QString& filename, const ComponentContainer& container)
 {
     QJsonObject obj;
     obj.insert("formatVersion", currentComponentsFileVersion);
     QJsonArray orderArray;
-    for(auto component: container)
-    {
+    for (auto component : container) {
         orderArray.append(componentToJsonV1(component));
     }
     obj.insert("components", orderArray);
     QSaveFile outFile(filename);
-    if (!outFile.open(QFile::WriteOnly))
-    {
-        qCritical() << "Couldn't open" << outFile.fileName()
-                     << "for writing:" << outFile.errorString();
+    if (!outFile.open(QFile::WriteOnly)) {
+        qCCritical(instanceProfileC) << "Couldn't open" << outFile.fileName() << "for writing:" << outFile.errorString();
         return false;
     }
     auto data = QJsonDocument(obj).toJson(QJsonDocument::Indented);
-    if(outFile.write(data) != data.size())
-    {
-        qCritical() << "Couldn't write all the data into" << outFile.fileName()
-                     << "because:" << outFile.errorString();
+    if (outFile.write(data) != data.size()) {
+        qCCritical(instanceProfileC) << "Couldn't write all the data into" << outFile.fileName() << "because:" << outFile.errorString();
         return false;
     }
-    if(!outFile.commit())
-    {
-        qCritical() << "Couldn't save" << outFile.fileName()
-                     << "because:" << outFile.errorString();
+    if (!outFile.commit()) {
+        qCCritical(instanceProfileC) << "Couldn't save" << outFile.fileName() << "because:" << outFile.errorString();
     }
     return true;
 }
 
 // Read the given file into component containers
-static bool loadPackProfile(PackProfile * parent, const QString & filename, const QString & componentJsonPattern, ComponentContainer & container)
+static bool loadPackProfile(PackProfile* parent,
+                            const QString& filename,
+                            const QString& componentJsonPattern,
+                            ComponentContainer& container)
 {
     QFile componentsFile(filename);
-    if (!componentsFile.exists())
-    {
-        qWarning() << "Components file doesn't exist. This should never happen.";
+    if (!componentsFile.exists()) {
+        qCWarning(instanceProfileC) << "Components file" << filename << "doesn't exist. This should never happen.";
         return false;
     }
-    if (!componentsFile.open(QFile::ReadOnly))
-    {
-        qCritical() << "Couldn't open" << componentsFile.fileName()
-                     << " for reading:" << componentsFile.errorString();
-        qWarning() << "Ignoring overriden order";
+    if (!componentsFile.open(QFile::ReadOnly)) {
+        qCCritical(instanceProfileC) << "Couldn't open" << componentsFile.fileName() << " for reading:" << componentsFile.errorString();
+        qCWarning(instanceProfileC) << "Ignoring overridden order";
         return false;
     }
 
     // and it's valid JSON
     QJsonParseError error;
     QJsonDocument doc = QJsonDocument::fromJson(componentsFile.readAll(), &error);
-    if (error.error != QJsonParseError::NoError)
-    {
-        qCritical() << "Couldn't parse" << componentsFile.fileName() << ":" << error.errorString();
-        qWarning() << "Ignoring overriden order";
+    if (error.error != QJsonParseError::NoError) {
+        qCCritical(instanceProfileC) << "Couldn't parse" << componentsFile.fileName() << ":" << error.errorString();
+        qCWarning(instanceProfileC) << "Ignoring overridden order";
         return false;
     }
 
     // and then read it and process it if all above is true.
-    try
-    {
+    try {
         auto obj = Json::requireObject(doc);
         // check order file version.
         auto version = Json::requireInteger(obj.value("formatVersion"));
-        if (version != currentComponentsFileVersion)
-        {
-            throw JSONValidationError(QObject::tr("Invalid component file version, expected %1")
-                                          .arg(currentComponentsFileVersion));
+        if (version != currentComponentsFileVersion) {
+            throw JSONValidationError(QObject::tr("Invalid component file version, expected %1").arg(currentComponentsFileVersion));
         }
         auto orderArray = Json::requireArray(obj.value("components"));
-        for(auto item: orderArray)
-        {
-            auto obj = Json::requireObject(item, "Component must be an object.");
-            container.append(componentFromJsonV1(parent, componentJsonPattern, obj));
+        for (auto item : orderArray) {
+            auto comp_obj = Json::requireObject(item, "Component must be an object.");
+            container.append(componentFromJsonV1(parent, componentJsonPattern, comp_obj));
         }
-    }
-    catch (const JSONValidationError &err)
-    {
-        qCritical() << "Couldn't parse" << componentsFile.fileName() << ": bad file format";
+    } catch ([[maybe_unused]] const JSONValidationError& err) {
+        qCCritical(instanceProfileC) << "Couldn't parse" << componentsFile.fileName() << ": bad file format";
         container.clear();
         return false;
     }
@@ -244,8 +225,7 @@ static bool loadPackProfile(PackProfile * parent, const QString & filename, cons
 
 void PackProfile::saveNow()
 {
-    if(saveIsScheduled())
-    {
+    if (saveIsScheduled()) {
         d->m_saveTimer.stop();
         save_internal();
     }
@@ -264,15 +244,13 @@ void PackProfile::buildingFromScratch()
 
 void PackProfile::scheduleSave()
 {
-    if(!d->loaded)
-    {
-        qDebug() << "Component list should never save if it didn't successfully load, instance:" << d->m_instance->name();
+    if (!d->loaded) {
+        qDebug() << d->m_instance->name() << "|" << "Component list should never save if it didn't successfully load";
         return;
     }
-    if(!d->dirty)
-    {
+    if (!d->dirty) {
         d->dirty = true;
-        qDebug() << "Component list save is scheduled for" << d->m_instance->name();
+        qDebug() << d->m_instance->name() << "|" << "Component list save is scheduled";
     }
     d->m_saveTimer.start();
 }
@@ -299,7 +277,7 @@ QString PackProfile::patchFilePathForUid(const QString& uid) const
 
 void PackProfile::save_internal()
 {
-    qDebug() << "Component list save performed now for" << d->m_instance->name();
+    qDebug() << d->m_instance->name() << "|" << "Component list save performed now";
     auto filename = componentsFilePath();
     savePackProfile(filename, d->components);
     d->dirty = false;
@@ -311,27 +289,21 @@ bool PackProfile::load()
 
     // load the new component list and swap it with the current one...
     ComponentContainer newComponents;
-    if(!loadPackProfile(this, filename, patchesPattern(), newComponents))
-    {
-        qCritical() << "Failed to load the component config for instance" << d->m_instance->name();
+    if (!loadPackProfile(this, filename, patchesPattern(), newComponents)) {
+        qCritical() << d->m_instance->name() << "|" << "Failed to load the component config";
         return false;
-    }
-    else
-    {
+    } else {
         // FIXME: actually use fine-grained updates, not this...
         beginResetModel();
         // disconnect all the old components
-        for(auto component: d->components)
-        {
+        for (auto component : d->components) {
             disconnect(component.get(), &Component::dataChanged, this, &PackProfile::componentDataChanged);
         }
         d->components.clear();
         d->componentIndex.clear();
-        for(auto component: newComponents)
-        {
-            if(d->componentIndex.contains(component->m_uid))
-            {
-                qWarning() << "Ignoring duplicate component entry" << component->m_uid;
+        for (auto component : newComponents) {
+            if (d->componentIndex.contains(component->m_uid)) {
+                qWarning() << d->m_instance->name() << "|" << "Ignoring duplicate component entry" << component->m_uid;
                 continue;
             }
             connect(component.get(), &Component::dataChanged, this, &PackProfile::componentDataChanged);
@@ -347,8 +319,7 @@ bool PackProfile::load()
 void PackProfile::reload(Net::Mode netmode)
 {
     // Do not reload when the update/resolve task is running. It is in control.
-    if(d->m_updateTask)
-    {
+    if (d->m_updateTask) {
         return;
     }
 
@@ -358,8 +329,7 @@ void PackProfile::reload(Net::Mode netmode)
     // FIXME: differentiate when a reapply is required by propagating state from components
     invalidateLaunchProfile();
 
-    if(load())
-    {
+    if (load()) {
         resolve(netmode);
     }
 }
@@ -375,21 +345,20 @@ void PackProfile::resolve(Net::Mode netmode)
     d->m_updateTask.reset(updateTask);
     connect(updateTask, &ComponentUpdateTask::succeeded, this, &PackProfile::updateSucceeded);
     connect(updateTask, &ComponentUpdateTask::failed, this, &PackProfile::updateFailed);
-    connect(updateTask, &ComponentUpdateTask::aborted, this, [this]{ updateFailed(tr("Aborted")); });
+    connect(updateTask, &ComponentUpdateTask::aborted, this, [this] { updateFailed(tr("Aborted")); });
     d->m_updateTask->start();
 }
 
-
 void PackProfile::updateSucceeded()
 {
-    qDebug() << "Component list update/resolve task succeeded for" << d->m_instance->name();
+    qCDebug(instanceProfileC) << d->m_instance->name() << "|" << "Component list update/resolve task succeeded";
     d->m_updateTask.reset();
     invalidateLaunchProfile();
 }
 
 void PackProfile::updateFailed(const QString& error)
 {
-    qDebug() << "Component list update/resolve task failed for" << d->m_instance->name() << "Reason:" << error;
+    qCDebug(instanceProfileC) << d->m_instance->name() << "|" << "Component list update/resolve task failed " << "Reason:" << error;
     d->m_updateTask.reset();
     invalidateLaunchProfile();
 }
@@ -404,17 +373,15 @@ void PackProfile::appendComponent(ComponentPtr component)
 void PackProfile::insertComponent(size_t index, ComponentPtr component)
 {
     auto id = component->getID();
-    if(id.isEmpty())
-    {
-        qWarning() << "Attempt to add a component with empty ID!";
+    if (id.isEmpty()) {
+        qCWarning(instanceProfileC) << d->m_instance->name() << "|" << "Attempt to add a component with empty ID!";
         return;
     }
-    if(d->componentIndex.contains(id))
-    {
-        qWarning() << "Attempt to add a component that is already present!";
+    if (d->componentIndex.contains(id)) {
+        qCWarning(instanceProfileC) << d->m_instance->name() << "|" << "Attempt to add a component that is already present!";
         return;
     }
-    beginInsertRows(QModelIndex(), index, index);
+    beginInsertRows(QModelIndex(), static_cast<int>(index), static_cast<int>(index));
     d->components.insert(index, component);
     d->componentIndex[id] = component;
     endInsertRows();
@@ -424,42 +391,38 @@ void PackProfile::insertComponent(size_t index, ComponentPtr component)
 
 void PackProfile::componentDataChanged()
 {
-    auto objPtr = qobject_cast<Component *>(sender());
-    if(!objPtr)
-    {
-        qWarning() << "PackProfile got dataChenged signal from a non-Component!";
+    auto objPtr = qobject_cast<Component*>(sender());
+    if (!objPtr) {
+        qCWarning(instanceProfileC) << d->m_instance->name() << "|" << "PackProfile got dataChanged signal from a non-Component!";
         return;
     }
-    if(objPtr->getID() == "net.minecraft") {
+    if (objPtr->getID() == "net.minecraft") {
         emit minecraftChanged();
     }
     // figure out which one is it... in a seriously dumb way.
     int index = 0;
-    for (auto component: d->components)
-    {
-        if(component.get() == objPtr)
-        {
+    for (auto component : d->components) {
+        if (component.get() == objPtr) {
             emit dataChanged(createIndex(index, 0), createIndex(index, columnCount(QModelIndex()) - 1));
             scheduleSave();
             return;
         }
         index++;
     }
-    qWarning() << "PackProfile got dataChenged signal from a Component which does not belong to it!";
+    qCWarning(instanceProfileC) << d->m_instance->name() << "|"
+                                << "PackProfile got dataChanged signal from a Component which does not belong to it!";
 }
 
 bool PackProfile::remove(const int index)
 {
     auto patch = getComponent(index);
-    if (!patch->isRemovable())
-    {
-        qWarning() << "Patch" << patch->getID() << "is non-removable";
+    if (!patch->isRemovable()) {
+        qCWarning(instanceProfileC) << d->m_instance->name() << "|" << "Patch" << patch->getID() << "is non-removable";
         return false;
     }
 
-    if(!removeComponent_internal(patch))
-    {
-        qCritical() << "Patch" << patch->getID() << "could not be removed";
+    if (!removeComponent_internal(patch)) {
+        qCCritical(instanceProfileC) << d->m_instance->name() << "|" << "Patch" << patch->getID() << "could not be removed";
         return false;
     }
 
@@ -472,13 +435,11 @@ bool PackProfile::remove(const int index)
     return true;
 }
 
-bool PackProfile::remove(const QString id)
+bool PackProfile::remove(const QString& id)
 {
     int i = 0;
-    for (auto patch : d->components)
-    {
-        if (patch->getID() == id)
-        {
+    for (auto patch : d->components) {
+        if (patch->getID() == id) {
             return remove(i);
         }
         i++;
@@ -489,14 +450,12 @@ bool PackProfile::remove(const QString id)
 bool PackProfile::customize(int index)
 {
     auto patch = getComponent(index);
-    if (!patch->isCustomizable())
-    {
-        qDebug() << "Patch" << patch->getID() << "is not customizable";
+    if (!patch->isCustomizable()) {
+        qCDebug(instanceProfileC) << d->m_instance->name() << "|" << "Patch" << patch->getID() << "is not customizable";
         return false;
     }
-    if(!patch->customize())
-    {
-        qCritical() << "Patch" << patch->getID() << "could not be customized";
+    if (!patch->customize()) {
+        qCCritical(instanceProfileC) << d->m_instance->name() << "|" << "Patch" << patch->getID() << "could not be customized";
         return false;
     }
     invalidateLaunchProfile();
@@ -507,14 +466,12 @@ bool PackProfile::customize(int index)
 bool PackProfile::revertToBase(int index)
 {
     auto patch = getComponent(index);
-    if (!patch->isRevertible())
-    {
-        qDebug() << "Patch" << patch->getID() << "is not revertible";
+    if (!patch->isRevertible()) {
+        qCDebug(instanceProfileC) << d->m_instance->name() << "|" << "Patch" << patch->getID() << "is not revertible";
         return false;
     }
-    if(!patch->revert())
-    {
-        qCritical() << "Patch" << patch->getID() << "could not be reverted";
+    if (!patch->revert()) {
+        qCCritical(instanceProfileC) << d->m_instance->name() << "|" << "Patch" << patch->getID() << "could not be reverted";
         return false;
     }
     invalidateLaunchProfile();
@@ -522,26 +479,24 @@ bool PackProfile::revertToBase(int index)
     return true;
 }
 
-ComponentPtr PackProfile::getComponent(const QString &id)
+ComponentPtr PackProfile::getComponent(const QString& id)
 {
     auto iter = d->componentIndex.find(id);
-    if (iter == d->componentIndex.end())
-    {
+    if (iter == d->componentIndex.end()) {
         return nullptr;
     }
     return (*iter);
 }
 
-ComponentPtr PackProfile::getComponent(int index)
+ComponentPtr PackProfile::getComponent(size_t index)
 {
-    if(index < 0 || index >= d->components.size())
-    {
+    if (index >= static_cast<size_t>(d->components.size())) {
         return nullptr;
     }
     return d->components[index];
 }
 
-QVariant PackProfile::data(const QModelIndex &index, int role) const
+QVariant PackProfile::data(const QModelIndex& index, int role) const
 {
     if (!index.isValid())
         return QVariant();
@@ -554,79 +509,58 @@ QVariant PackProfile::data(const QModelIndex &index, int role) const
 
     auto patch = d->components.at(row);
 
-    switch (role)
-    {
-    case Qt::CheckStateRole:
-    {
-        switch (column)
-        {
-            case NameColumn: {
-                return patch->isEnabled() ? Qt::Checked : Qt::Unchecked;
-            }
-            default:
-                return QVariant();
-        }
-    }
-    case Qt::DisplayRole:
-    {
-        switch (column)
-        {
-        case NameColumn:
-            return patch->getName();
-        case VersionColumn:
-        {
-            if(patch->isCustom())
-            {
-                return QString("%1 (Custom)").arg(patch->getVersion());
-            }
-            else
-            {
-                return patch->getVersion();
-            }
-        }
-        default:
-            return QVariant();
-        }
-    }
-    case Qt::DecorationRole:
-    {
-        switch(column)
-        {
-        case NameColumn:
-        {
-            auto severity = patch->getProblemSeverity();
-            switch (severity)
-            {
-                case ProblemSeverity::Warning:
-                    return "warning";
-                case ProblemSeverity::Error:
-                    return "error";
+    switch (role) {
+        case Qt::CheckStateRole: {
+            switch (column) {
+                case NameColumn: {
+                    return patch->isEnabled() ? Qt::Checked : Qt::Unchecked;
+                }
                 default:
                     return QVariant();
             }
         }
-        default:
-        {
+        case Qt::DisplayRole: {
+            switch (column) {
+                case NameColumn:
+                    return patch->getName();
+                case VersionColumn: {
+                    if (patch->isCustom()) {
+                        return QString("%1 (Custom)").arg(patch->getVersion());
+                    } else {
+                        return patch->getVersion();
+                    }
+                }
+                default:
+                    return QVariant();
+            }
+        }
+        case Qt::DecorationRole: {
+            if (column == NameColumn) {
+                auto severity = patch->getProblemSeverity();
+                switch (severity) {
+                    case ProblemSeverity::Warning:
+                        return "warning";
+                    case ProblemSeverity::Error:
+                        return "error";
+                    default:
+                        return QVariant();
+                }
+            }
             return QVariant();
         }
-        }
-    }
     }
     return QVariant();
 }
 
-bool PackProfile::setData(const QModelIndex& index, const QVariant& value, int role)
+bool PackProfile::setData(const QModelIndex& index, [[maybe_unused]] const QVariant& value, int role)
 {
-    if (!index.isValid() || index.row() < 0 || index.row() >= rowCount(index.parent()))
-    {
+    if (!index.isValid() || index.row() < 0 || index.row() >= rowCount(index.parent())) {
         return false;
     }
 
-    if (role == Qt::CheckStateRole)
-    {
+    if (role == Qt::CheckStateRole) {
         auto component = d->components[index.row()];
-        if (component->setEnabled(!component->isEnabled()))
-        {
+        if (component->setEnabled(!component->isEnabled())) {
             return true;
         }
     }
@@ -635,18 +569,15 @@ bool PackProfile::setData(const QModelIndex& index, const QVariant& value, int r
 
 QVariant PackProfile::headerData(int section, Qt::Orientation orientation, int role) const
 {
-    if (orientation == Qt::Horizontal)
-    {
-        if (role == Qt::DisplayRole)
-        {
-            switch (section)
-            {
-            case NameColumn:
-                return tr("Name");
-            case VersionColumn:
-                return tr("Version");
-            default:
-                return QVariant();
+    if (orientation == Qt::Horizontal) {
+        if (role == Qt::DisplayRole) {
+            switch (section) {
+                case NameColumn:
+                    return tr("Name");
+                case VersionColumn:
+                    return tr("Version");
+                default:
+                    return QVariant();
             }
         }
     }
@@ -654,7 +585,7 @@ QVariant PackProfile::headerData(int section, Qt::Orientation orientation, int r
 }
 
 // FIXME: zero precision mess
-Qt::ItemFlags PackProfile::flags(const QModelIndex &index) const
+Qt::ItemFlags PackProfile::flags(const QModelIndex& index) const
 {
     if (!index.isValid()) {
         return Qt::NoItemFlags;
@@ -670,19 +601,18 @@ Qt::ItemFlags PackProfile::flags(const QModelIndex &index) const
 
     auto patch = d->components.at(row);
     // TODO: this will need fine-tuning later...
-    if(patch->canBeDisabled() && !d->interactionDisabled)
-    {
+    if (patch->canBeDisabled() && !d->interactionDisabled) {
         outFlags |= Qt::ItemIsUserCheckable;
     }
     return outFlags;
 }
 
-int PackProfile::rowCount(const QModelIndex &parent) const
+int PackProfile::rowCount(const QModelIndex& parent) const
 {
     return parent.isValid() ? 0 : d->components.size();
 }
 
-int PackProfile::columnCount(const QModelIndex &parent) const
+int PackProfile::columnCount(const QModelIndex& parent) const
 {
     return parent.isValid() ? 0 : NUM_COLUMNS;
 }
@@ -690,12 +620,9 @@ int PackProfile::columnCount(const QModelIndex &parent) const
 void PackProfile::move(const int index, const MoveDirection direction)
 {
     int theirIndex;
-    if (direction == MoveUp)
-    {
+    if (direction == MoveUp) {
         theirIndex = index - 1;
-    }
-    else
-    {
+    } else {
         theirIndex = index + 1;
     }
 
@@ -712,8 +639,7 @@ void PackProfile::move(const int index, const MoveDirection direction)
     auto from = getComponent(index);
     auto to = getComponent(theirIndex);
 
-    if (!from || !to || !to->isMoveable() || !from->isMoveable())
-    {
+    if (!from || !to || !to->isMoveable() || !from->isMoveable()) {
         return;
     }
     beginMoveRows(QModelIndex(), index, index, QModelIndex(), togap);
@@ -758,7 +684,8 @@ bool PackProfile::installComponents(QStringList selectedFiles)
         const QString target = FS::PathCombine(patchDir, versionFile->uid + ".json");
 
         if (!QFile::copy(source, target)) {
-            qWarning() << "Component" << source << "could not be copied to target" << target;
+            qCWarning(instanceProfileC) << d->m_instance->name() << "|" << "Component" << source << "could not be copied to target"
+                                        << target;
             result = false;
             continue;
         }
@@ -781,8 +708,7 @@ void PackProfile::installAgents(QStringList selectedFiles)
 bool PackProfile::installEmpty(const QString& uid, const QString& name)
 {
     QString patchDir = FS::PathCombine(d->m_instance->instanceRoot(), "patches");
-    if(!FS::ensureFolderPathExists(patchDir))
-    {
+    if (!FS::ensureFolderPathExists(patchDir)) {
         return false;
     }
     auto f = std::make_shared<VersionFile>();
@@ -791,10 +717,9 @@ bool PackProfile::installEmpty(const QString& uid, const QString& name)
     f->version = "1";
     QString patchFileName = FS::PathCombine(patchDir, uid + ".json");
     QFile file(patchFileName);
-    if (!file.open(QFile::WriteOnly))
-    {
-        qCritical() << "Error opening" << file.fileName()
-                    << "for reading:" << file.errorString();
+    if (!file.open(QFile::WriteOnly)) {
+        qCCritical(instanceProfileC) << d->m_instance->name() << "|" << "Error opening" << file.fileName()
+                                     << "for reading:" << file.errorString();
         return false;
     }
     file.write(OneSixVersionFormat::versionFileToJson(f).toJson());
@@ -811,32 +736,28 @@ bool PackProfile::removeComponent_internal(ComponentPtr patch)
     bool ok = true;
     // first, remove the patch file. this ensures it's not used anymore
     auto fileName = patch->getFilename();
-    if(fileName.size())
-    {
+    if (fileName.size()) {
         QFile patchFile(fileName);
-        if(patchFile.exists() && !patchFile.remove())
-        {
-            qCritical() << "File" << fileName << "could not be removed because:" << patchFile.errorString();
+        if (patchFile.exists() && !patchFile.remove()) {
+            qCCritical(instanceProfileC) << d->m_instance->name() << "|" << "File" << fileName
+                                         << "could not be removed because:" << patchFile.errorString();
             return false;
         }
     }
 
     // FIXME: we need a generic way of removing local resources, not just jar mods...
-    auto preRemoveJarMod = [&](LibraryPtr jarMod) -> bool
-    {
-        if (!jarMod->isLocal())
-        {
+    auto preRemoveJarMod = [&](LibraryPtr jarMod) -> bool {
+        if (!jarMod->isLocal()) {
             return true;
         }
         QStringList jar, temp1, temp2, temp3;
         jarMod->getApplicableFiles(d->m_instance->runtimeContext(), jar, temp1, temp2, temp3, d->m_instance->jarmodsPath().absolutePath());
-        QFileInfo finfo (jar[0]);
-        if(finfo.exists())
-        {
+        QFileInfo finfo(jar[0]);
+        if (finfo.exists()) {
             QFile jarModFile(jar[0]);
-            if(!jarModFile.remove())
-            {
-                qCritical() << "File" << jar[0] << "could not be removed because:" << jarModFile.errorString();
+            if (!jarModFile.remove()) {
+                qCCritical(instanceProfileC) << d->m_instance->name() << "|" << "File" << jar[0]
+                                             << "could not be removed because:" << jarModFile.errorString();
                 return false;
             }
             return true;
@@ -845,11 +766,9 @@ bool PackProfile::removeComponent_internal(ComponentPtr patch)
     };
 
     auto vFile = patch->getVersionFile();
-    if(vFile)
-    {
-        auto &jarMods = vFile->jarMods;
-        for(auto &jarmod: jarMods)
-        {
+    if (vFile) {
+        auto& jarMods = vFile->jarMods;
+        for (auto& jarmod : jarMods) {
             ok &= preRemoveJarMod(jarmod);
         }
     }
@@ -859,18 +778,15 @@ bool PackProfile::removeComponent_internal(ComponentPtr patch)
 bool PackProfile::installJarMods_internal(QStringList filepaths)
 {
     QString patchDir = FS::PathCombine(d->m_instance->instanceRoot(), "patches");
-    if(!FS::ensureFolderPathExists(patchDir))
-    {
+    if (!FS::ensureFolderPathExists(patchDir)) {
         return false;
     }
 
-    if (!FS::ensureFolderPathExists(d->m_instance->jarModsDir()))
-    {
+    if (!FS::ensureFolderPathExists(d->m_instance->jarModsDir())) {
         return false;
     }
 
-    for(auto filepath:filepaths)
-    {
+    for (auto filepath : filepaths) {
         QFileInfo sourceInfo(filepath);
         QString id = QUuid::createUuid().toString(QUuid::WithoutBraces);
         QString target_filename = id + ".jar";
@@ -881,8 +797,7 @@ bool PackProfile::installJarMods_internal(QStringList filepaths)
         QFileInfo targetInfo(finalPath);
         Q_ASSERT(!targetInfo.exists());
 
-        if (!QFile::copy(sourceInfo.absoluteFilePath(),QFileInfo(finalPath).absoluteFilePath()))
-        {
+        if (!QFile::copy(sourceInfo.absoluteFilePath(), QFileInfo(finalPath).absoluteFilePath())) {
             return false;
         }
 
@@ -898,10 +813,9 @@ bool PackProfile::installJarMods_internal(QStringList filepaths)
         QString patchFileName = FS::PathCombine(patchDir, target_id + ".json");
 
         QFile file(patchFileName);
-        if (!file.open(QFile::WriteOnly))
-        {
-            qCritical() << "Error opening" << file.fileName()
-                        << "for reading:" << file.errorString();
+        if (!file.open(QFile::WriteOnly)) {
+            qCCritical(instanceProfileC) << d->m_instance->name() << "|" << "Error opening" << file.fileName()
+                                         << "for reading:" << file.errorString();
             return false;
         }
         file.write(OneSixVersionFormat::versionFileToJson(f).toJson());
@@ -917,14 +831,12 @@ bool PackProfile::installJarMods_internal(QStringList filepaths)
 bool PackProfile::installCustomJar_internal(QString filepath)
 {
     QString patchDir = FS::PathCombine(d->m_instance->instanceRoot(), "patches");
-    if(!FS::ensureFolderPathExists(patchDir))
-    {
+    if (!FS::ensureFolderPathExists(patchDir)) {
         return false;
     }
 
     QString libDir = d->m_instance->getLocalLibraryPath();
-    if (!FS::ensureFolderPathExists(libDir))
-    {
+    if (!FS::ensureFolderPathExists(libDir)) {
         return false;
     }
 
@@ -936,15 +848,12 @@ bool PackProfile::installCustomJar_internal(QString filepath)
     QString finalPath = FS::PathCombine(libDir, target_filename);
 
     QFileInfo jarInfo(finalPath);
-    if (jarInfo.exists())
-    {
-        if(!QFile::remove(finalPath))
-        {
+    if (jarInfo.exists()) {
+        if (!FS::deletePath(finalPath)) {
             return false;
         }
     }
-    if (!QFile::copy(filepath, finalPath))
-    {
+    if (!QFile::copy(filepath, finalPath)) {
         return false;
     }
 
@@ -959,10 +868,9 @@ bool PackProfile::installCustomJar_internal(QString filepath)
     QString patchFileName = FS::PathCombine(patchDir, target_id + ".json");
 
     QFile file(patchFileName);
-    if (!file.open(QFile::WriteOnly))
-    {
-        qCritical() << "Error opening" << file.fileName()
-                    << "for reading:" << file.errorString();
+    if (!file.open(QFile::WriteOnly)) {
+        qCCritical(instanceProfileC) << d->m_instance->name() << "|" << "Error opening" << file.fileName()
+                                     << "for reading:" << file.errorString();
         return false;
     }
     file.write(OneSixVersionFormat::versionFileToJson(f).toJson());
@@ -1017,7 +925,8 @@ bool PackProfile::installAgents_internal(QStringList filepaths)
         QFile patchFile(FS::PathCombine(patchDir, targetId + ".json"));
 
         if (!patchFile.open(QFile::WriteOnly)) {
-            qCritical() << "Error opening" << patchFile.fileName() << "for reading:" << patchFile.errorString();
+            qCCritical(instanceProfileC) << d->m_instance->name() << "|" << "Error opening" << patchFile.fileName()
+                                         << "for reading:" << patchFile.errorString();
             return false;
         }
 
@@ -1035,21 +944,17 @@ bool PackProfile::installAgents_internal(QStringList filepaths)
 
 std::shared_ptr<LaunchProfile> PackProfile::getProfile() const
 {
-    if(!d->m_profile)
-    {
-        try
-        {
+    if (!d->m_profile) {
+        try {
             auto profile = std::make_shared<LaunchProfile>();
-            for(auto file: d->components)
-            {
-                qDebug() << "Applying" << file->getID() << (file->getProblemSeverity() == ProblemSeverity::Error ? "ERROR" : "GOOD");
+            for (auto file : d->components) {
+                qCDebug(instanceProfileC) << d->m_instance->name() << "|" << "Applying" << file->getID()
+                                          << (file->getProblemSeverity() == ProblemSeverity::Error ? "ERROR" : "GOOD");
                 file->applyTo(profile.get());
             }
             d->m_profile = profile;
-        }
-        catch (const Exception &error)
-        {
-            qWarning() << "Couldn't apply profile patches because: " << error.cause();
+        } catch (const Exception& error) {
+            qCWarning(instanceProfileC) << d->m_instance->name() << "|" << "Couldn't apply profile patches because: " << error.cause();
         }
     }
     return d->m_profile;
@@ -1058,20 +963,24 @@ std::shared_ptr<LaunchProfile> PackProfile::getProfile() const
 bool PackProfile::setComponentVersion(const QString& uid, const QString& version, bool important)
 {
     auto iter = d->componentIndex.find(uid);
-    if(iter != d->componentIndex.end())
-    {
+    if (iter != d->componentIndex.end()) {
         ComponentPtr component = *iter;
         // set existing
-        if(component->revert())
-        {
+        if (component->revert()) {
+            // set new version
+            auto oldVersion = component->getVersion();
             component->setVersion(version);
             component->setImportant(important);
+
+            if (important) {
+                component->setUpdateAction(UpdateAction{ UpdateActionImportantChanged{ oldVersion } });
+                resolve(Net::Mode::Online);
+            }
+
             return true;
         }
         return false;
-    }
-    else
-    {
+    } else {
         // add new
         auto component = makeShared<Component>(this, uid);
         component->m_version = version;
@@ -1084,8 +993,7 @@ bool PackProfile::setComponentVersion(const QString& uid, const QString& version
 QString PackProfile::getComponentVersion(const QString& uid) const
 {
     const auto iter = d->componentIndex.find(uid);
-    if (iter != d->componentIndex.end())
-    {
+    if (iter != d->componentIndex.end()) {
         return (*iter)->getVersion();
     }
     return QString();
@@ -1093,26 +1001,26 @@ QString PackProfile::getComponentVersion(const QString& uid) const
 
 void PackProfile::disableInteraction(bool disable)
 {
-    if(d->interactionDisabled != disable) {
+    if (d->interactionDisabled != disable) {
         d->interactionDisabled = disable;
         auto size = d->components.size();
-        if(size) {
+        if (size) {
             emit dataChanged(index(0), index(size - 1));
         }
     }
 }
 
-std::optional<ResourceAPI::ModLoaderTypes> PackProfile::getModLoaders()
+std::optional<ModPlatform::ModLoaderTypes> PackProfile::getModLoaders()
 {
-    ResourceAPI::ModLoaderTypes result;
+    ModPlatform::ModLoaderTypes result;
     bool has_any_loader = false;
 
-    QMapIterator<QString, ResourceAPI::ModLoaderType> i(modloaderMapping);
+    QMapIterator<QString, ModloaderMapEntry> i(Component::KNOWN_MODLOADERS);
 
     while (i.hasNext()) {
         i.next();
         if (auto c = getComponent(i.key()); c != nullptr && c->isEnabled()) {
-            result |= i.value();
+            result |= i.value().type;
             has_any_loader = true;
         }
     }
@@ -1121,3 +1029,37 @@ std::optional<ResourceAPI::ModLoaderTypes> PackProfile::getModLoaders()
         return {};
     return result;
 }
+
+std::optional<ModPlatform::ModLoaderTypes> PackProfile::getSupportedModLoaders()
+{
+    auto loadersOpt = getModLoaders();
+    if (!loadersOpt.has_value())
+        return loadersOpt;
+    auto loaders = loadersOpt.value();
+    // TODO: remove this or add version condition once Quilt drops official Fabric support
+    if (loaders & ModPlatform::Quilt)
+        loaders |= ModPlatform::Fabric;
+    if (getComponentVersion("net.minecraft") == "1.20.1" && (loaders & ModPlatform::NeoForge))
+        loaders |= ModPlatform::Forge;
+    return loaders;
+}
+
+QList<ModPlatform::ModLoaderType> PackProfile::getModLoadersList()
+{
+    QList<ModPlatform::ModLoaderType> result;
+    for (auto c : d->components) {
+        if (c->isEnabled() && Component::KNOWN_MODLOADERS.contains(c->getID())) {
+            result.append(Component::KNOWN_MODLOADERS[c->getID()].type);
+        }
+    }
+
+    // TODO: remove this or add version condition once Quilt drops official Fabric support
+    if (result.contains(ModPlatform::Quilt) && !result.contains(ModPlatform::Fabric)) {
+        result.append(ModPlatform::Fabric);
+    }
+    if (getComponentVersion("net.minecraft") == "1.20.1" && result.contains(ModPlatform::NeoForge) &&
+        !result.contains(ModPlatform::Forge)) {
+        result.append(ModPlatform::Forge);
+    }
+    return result;
+}
