@@ -36,54 +36,53 @@
  */
 
 #include "ScreenshotsPage.h"
+#include "BuildConfig.h"
 #include "ui_ScreenshotsPage.h"
 
-#include <QModelIndex>
-#include <QMutableListIterator>
-#include <QMap>
-#include <QSet>
+#include <QClipboard>
+#include <QEvent>
 #include <QFileIconProvider>
 #include <QFileSystemModel>
-#include <QStyledItemDelegate>
-#include <QLineEdit>
-#include <QEvent>
-#include <QPainter>
-#include <QClipboard>
 #include <QKeyEvent>
+#include <QLineEdit>
+#include <QMap>
 #include <QMenu>
+#include <QModelIndex>
+#include <QMutableListIterator>
+#include <QPainter>
 #include <QRegularExpression>
+#include <QSet>
+#include <QStyledItemDelegate>
 
 #include <Application.h>
 
-#include "ui/dialogs/ProgressDialog.h"
 #include "ui/dialogs/CustomMessageBox.h"
+#include "ui/dialogs/ProgressDialog.h"
 
 #include "net/NetJob.h"
-#include "screenshots/ImgurUpload.h"
 #include "screenshots/ImgurAlbumCreation.h"
+#include "screenshots/ImgurUpload.h"
 #include "tasks/SequentialTask.h"
 
-#include "RWStorage.h"
-#include <FileSystem.h>
 #include <DesktopServices.h>
+#include <FileSystem.h>
+#include "RWStorage.h"
 
-typedef RWStorage<QString, QIcon> SharedIconCache;
-typedef std::shared_ptr<SharedIconCache> SharedIconCachePtr;
+using SharedIconCache = RWStorage<QString, QIcon>;
+using SharedIconCachePtr = std::shared_ptr<SharedIconCache>;
 
-class ThumbnailingResult : public QObject
-{
+class ThumbnailingResult : public QObject {
     Q_OBJECT
-public slots:
-    inline void emitResultsReady(const QString &path) { emit resultsReady(path); }
-    inline void emitResultsFailed(const QString &path) { emit resultsFailed(path); }
-signals:
-    void resultsReady(const QString &path);
-    void resultsFailed(const QString &path);
+   public slots:
+    inline void emitResultsReady(const QString& path) { emit resultsReady(path); }
+    inline void emitResultsFailed(const QString& path) { emit resultsFailed(path); }
+   signals:
+    void resultsReady(const QString& path);
+    void resultsFailed(const QString& path);
 };
 
-class ThumbnailRunnable : public QRunnable
-{
-public:
+class ThumbnailRunnable : public QRunnable {
+   public:
     ThumbnailRunnable(QString path, SharedIconCachePtr cache)
     {
         m_path = path;
@@ -96,37 +95,30 @@ public:
             return;
         if ((info.suffix().compare("png", Qt::CaseInsensitive) != 0))
             return;
-        int tries = 5;
-        while (tries)
-        {
-            if (!m_cache->stale(m_path))
-                return;
-            QImage image(m_path);
-            if (image.isNull())
-            {
-                QThread::msleep(500);
-                tries--;
-                continue;
-            }
-            QImage small;
-            if (image.width() > image.height())
-                small = image.scaledToWidth(512).scaledToWidth(256, Qt::SmoothTransformation);
-            else
-                small = image.scaledToHeight(512).scaledToHeight(256, Qt::SmoothTransformation);
-            QPoint offset((256 - small.width()) / 2, (256 - small.height()) / 2);
-            QImage square(QSize(256, 256), QImage::Format_ARGB32);
-            square.fill(Qt::transparent);
-
-            QPainter painter(&square);
-            painter.drawImage(offset, small);
-            painter.end();
-
-            QIcon icon(QPixmap::fromImage(square));
-            m_cache->add(m_path, icon);
-            m_resultEmitter.emitResultsReady(m_path);
+        if (!m_cache->stale(m_path))
+            return;
+        QImage image(m_path);
+        if (image.isNull()) {
+            m_resultEmitter.emitResultsFailed(m_path);
+            qDebug() << "Error loading screenshot: " + m_path + ". Perhaps too large?";
             return;
         }
-        m_resultEmitter.emitResultsFailed(m_path);
+        QImage small;
+        if (image.width() > image.height())
+            small = image.scaledToWidth(512).scaledToWidth(256, Qt::SmoothTransformation);
+        else
+            small = image.scaledToHeight(512).scaledToHeight(256, Qt::SmoothTransformation);
+        QPoint offset((256 - small.width()) / 2, (256 - small.height()) / 2);
+        QImage square(QSize(256, 256), QImage::Format_ARGB32);
+        square.fill(Qt::transparent);
+
+        QPainter painter(&square);
+        painter.drawImage(offset, small);
+        painter.end();
+
+        QIcon icon(QPixmap::fromImage(square));
+        m_cache->add(m_path, icon);
+        m_resultEmitter.emitResultsReady(m_path);
     }
     QString m_path;
     SharedIconCachePtr m_cache;
@@ -135,54 +127,50 @@ public:
 
 // this is about as elegant and well written as a bag of bricks with scribbles done by insane
 // asylum patients.
-class FilterModel : public QIdentityProxyModel
-{
+class FilterModel : public QIdentityProxyModel {
     Q_OBJECT
-public:
-    explicit FilterModel(QObject *parent = 0) : QIdentityProxyModel(parent)
+   public:
+    explicit FilterModel(QObject* parent = 0) : QIdentityProxyModel(parent)
     {
         m_thumbnailingPool.setMaxThreadCount(4);
         m_thumbnailCache = std::make_shared<SharedIconCache>();
         m_thumbnailCache->add("placeholder", APPLICATION->getThemedIcon("screenshot-placeholder"));
         connect(&watcher, SIGNAL(fileChanged(QString)), SLOT(fileChanged(QString)));
-        // FIXME: the watched file set is not updated when files are removed
     }
-    virtual ~FilterModel() { m_thumbnailingPool.waitForDone(500); }
-    virtual QVariant data(const QModelIndex &proxyIndex, int role = Qt::DisplayRole) const
+    virtual ~FilterModel()
+    {
+        m_thumbnailingPool.clear();
+        if (!m_thumbnailingPool.waitForDone(500))
+            qDebug() << "Thumbnail pool took longer than 500ms to finish";
+    }
+    virtual QVariant data(const QModelIndex& proxyIndex, int role = Qt::DisplayRole) const
     {
         auto model = sourceModel();
         if (!model)
             return QVariant();
-        if (role == Qt::DisplayRole || role == Qt::EditRole)
-        {
+        if (role == Qt::DisplayRole || role == Qt::EditRole) {
             QVariant result = sourceModel()->data(mapToSource(proxyIndex), role);
             return result.toString().remove(QRegularExpression("\\.png$"));
         }
-        if (role == Qt::DecorationRole)
-        {
-            QVariant result =
-                sourceModel()->data(mapToSource(proxyIndex), QFileSystemModel::FilePathRole);
+        if (role == Qt::DecorationRole) {
+            QVariant result = sourceModel()->data(mapToSource(proxyIndex), QFileSystemModel::FilePathRole);
             QString filePath = result.toString();
             QIcon temp;
-            if (!watched.contains(filePath))
-            {
-                ((QFileSystemWatcher &)watcher).addPath(filePath);
-                ((QSet<QString> &)watched).insert(filePath);
+            if (!watched.contains(filePath)) {
+                ((QFileSystemWatcher&)watcher).addPath(filePath);
+                ((QSet<QString>&)watched).insert(filePath);
             }
-            if (m_thumbnailCache->get(filePath, temp))
-            {
+            if (m_thumbnailCache->get(filePath, temp)) {
                 return temp;
             }
-            if (!m_failed.contains(filePath))
-            {
-                ((FilterModel *)this)->thumbnailImage(filePath);
+            if (!m_failed.contains(filePath)) {
+                ((FilterModel*)this)->thumbnailImage(filePath);
             }
             return (m_thumbnailCache->get("placeholder"));
         }
         return sourceModel()->data(mapToSource(proxyIndex), role);
     }
-    virtual bool setData(const QModelIndex &index, const QVariant &value,
-                         int role = Qt::EditRole)
+    virtual bool setData(const QModelIndex& index, const QVariant& value, int role = Qt::EditRole)
     {
         auto model = sourceModel();
         if (!model)
@@ -192,35 +180,35 @@ public:
         // FIXME: this is a workaround for a bug in QFileSystemModel, where it doesn't
         // sort after renames
         {
-            ((QFileSystemModel *)model)->setNameFilterDisables(true);
-            ((QFileSystemModel *)model)->setNameFilterDisables(false);
+            ((QFileSystemModel*)model)->setNameFilterDisables(true);
+            ((QFileSystemModel*)model)->setNameFilterDisables(false);
         }
         return model->setData(mapToSource(index), value.toString() + ".png", role);
     }
 
-private:
+   private:
     void thumbnailImage(QString path)
     {
         auto runnable = new ThumbnailRunnable(path, m_thumbnailCache);
-        connect(&(runnable->m_resultEmitter), SIGNAL(resultsReady(QString)),
-                SLOT(thumbnailReady(QString)));
-        connect(&(runnable->m_resultEmitter), SIGNAL(resultsFailed(QString)),
-                SLOT(thumbnailFailed(QString)));
-        ((QThreadPool &)m_thumbnailingPool).start(runnable);
+        connect(&(runnable->m_resultEmitter), SIGNAL(resultsReady(QString)), SLOT(thumbnailReady(QString)));
+        connect(&(runnable->m_resultEmitter), SIGNAL(resultsFailed(QString)), SLOT(thumbnailFailed(QString)));
+        ((QThreadPool&)m_thumbnailingPool).start(runnable);
     }
-private slots:
+   private slots:
     void thumbnailReady(QString path) { emit layoutChanged(); }
     void thumbnailFailed(QString path) { m_failed.insert(path); }
     void fileChanged(QString filepath)
     {
         m_thumbnailCache->setStale(filepath);
-        thumbnailImage(filepath);
         // reinsert the path...
         watcher.removePath(filepath);
-        watcher.addPath(filepath);
+        if (QFile::exists(filepath)) {
+            watcher.addPath(filepath);
+            thumbnailImage(filepath);
+        }
     }
 
-private:
+   private:
     SharedIconCachePtr m_thumbnailCache;
     QThreadPool m_thumbnailingPool;
     QSet<QString> m_failed;
@@ -228,18 +216,15 @@ private:
     QFileSystemWatcher watcher;
 };
 
-class CenteredEditingDelegate : public QStyledItemDelegate
-{
-public:
-    explicit CenteredEditingDelegate(QObject *parent = 0) : QStyledItemDelegate(parent) {}
+class CenteredEditingDelegate : public QStyledItemDelegate {
+   public:
+    explicit CenteredEditingDelegate(QObject* parent = 0) : QStyledItemDelegate(parent) {}
     virtual ~CenteredEditingDelegate() {}
-    virtual QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option,
-                                  const QModelIndex &index) const
+    virtual QWidget* createEditor(QWidget* parent, const QStyleOptionViewItem& option, const QModelIndex& index) const
     {
         auto widget = QStyledItemDelegate::createEditor(parent, option, index);
-        auto foo = dynamic_cast<QLineEdit *>(widget);
-        if (foo)
-        {
+        auto foo = dynamic_cast<QLineEdit*>(widget);
+        if (foo) {
             foo->setAlignment(Qt::AlignHCenter);
             foo->setFrame(true);
             foo->setMaximumWidth(192);
@@ -248,16 +233,20 @@ public:
     }
 };
 
-ScreenshotsPage::ScreenshotsPage(QString path, QWidget *parent)
-    : QMainWindow(parent), ui(new Ui::ScreenshotsPage)
+ScreenshotsPage::ScreenshotsPage(QString path, QWidget* parent) : QMainWindow(parent), ui(new Ui::ScreenshotsPage)
 {
     m_model.reset(new QFileSystemModel());
     m_filterModel.reset(new FilterModel());
     m_filterModel->setSourceModel(m_model.get());
     m_model->setFilter(QDir::Files);
     m_model->setReadOnly(false);
-    m_model->setNameFilters({"*.png"});
+    m_model->setNameFilters({ "*.png" });
     m_model->setNameFilterDisables(false);
+    // Sorts by modified date instead of creation date because that column is not available and would require subclassing, this should work
+    // considering screenshots aren't modified after creation.
+    constexpr int file_modified_column_index = 3;
+    m_model->sort(file_modified_column_index, Qt::DescendingOrder);
+
     m_folder = path;
     m_valid = FS::ensureFolderPathExists(m_folder);
 
@@ -279,31 +268,29 @@ ScreenshotsPage::ScreenshotsPage(QString path, QWidget *parent)
     connect(ui->listView, SIGNAL(activated(QModelIndex)), SLOT(onItemActivated(QModelIndex)));
 }
 
-bool ScreenshotsPage::eventFilter(QObject *obj, QEvent *evt)
+bool ScreenshotsPage::eventFilter(QObject* obj, QEvent* evt)
 {
     if (obj != ui->listView)
         return QWidget::eventFilter(obj, evt);
-    if (evt->type() != QEvent::KeyPress)
-    {
+    if (evt->type() != QEvent::KeyPress) {
         return QWidget::eventFilter(obj, evt);
     }
-    QKeyEvent *keyEvent = static_cast<QKeyEvent *>(evt);
+    QKeyEvent* keyEvent = static_cast<QKeyEvent*>(evt);
 
     if (keyEvent->matches(QKeySequence::Copy)) {
         on_actionCopy_File_s_triggered();
         return true;
     }
 
-    switch (keyEvent->key())
-    {
-    case Qt::Key_Delete:
-        on_actionDelete_triggered();
-        return true;
-    case Qt::Key_F2:
-        on_actionRename_triggered();
-        return true;
-    default:
-        break;
+    switch (keyEvent->key()) {
+        case Qt::Key_Delete:
+            on_actionDelete_triggered();
+            return true;
+        case Qt::Key_F2:
+            on_actionRename_triggered();
+            return true;
+        default:
+            break;
     }
     return QWidget::eventFilter(obj, evt);
 }
@@ -323,17 +310,17 @@ void ScreenshotsPage::ShowContextMenu(const QPoint& pos)
     auto menu = ui->toolBar->createContextMenu(this, tr("Context menu"));
 
     if (ui->listView->selectionModel()->selectedRows().size() > 1) {
-        menu->removeAction( ui->actionCopy_Image );
+        menu->removeAction(ui->actionCopy_Image);
     }
 
     menu->exec(ui->listView->mapToGlobal(pos));
     delete menu;
 }
 
-QMenu * ScreenshotsPage::createPopupMenu()
+QMenu* ScreenshotsPage::createPopupMenu()
 {
     QMenu* filteredMenu = QMainWindow::createPopupMenu();
-    filteredMenu->removeAction( ui->toolBar->toggleViewAction() );
+    filteredMenu->removeAction(ui->toolBar->toggleViewAction());
     return filteredMenu;
 }
 
@@ -342,17 +329,15 @@ void ScreenshotsPage::onItemActivated(QModelIndex index)
     if (!index.isValid())
         return;
     auto info = m_model->fileInfo(index);
-    QString fileName = info.absoluteFilePath();
-    DesktopServices::openFile(info.absoluteFilePath());
+    DesktopServices::openPath(info);
 }
 
-void ScreenshotsPage::onCurrentSelectionChanged(const QItemSelection &selected)
+void ScreenshotsPage::onCurrentSelectionChanged(const QItemSelection& selected)
 {
     bool allReadable = !selected.isEmpty();
     bool allWritable = !selected.isEmpty();
 
-    for (auto index : selected.indexes())
-    {
+    for (auto index : selected.indexes()) {
         if (!index.isValid())
             break;
         auto info = m_model->fileInfo(index);
@@ -371,7 +356,7 @@ void ScreenshotsPage::onCurrentSelectionChanged(const QItemSelection &selected)
 
 void ScreenshotsPage::on_actionView_Folder_triggered()
 {
-    DesktopServices::openDirectory(m_folder, true);
+    DesktopServices::openPath(m_folder, true);
 }
 
 void ScreenshotsPage::on_actionUpload_triggered()
@@ -380,16 +365,18 @@ void ScreenshotsPage::on_actionUpload_triggered()
     if (selection.isEmpty())
         return;
 
-
     QString text;
+    QUrl baseUrl(BuildConfig.IMGUR_BASE_URL);
     if (selection.size() > 1)
-        text = tr("You are about to upload %1 screenshots.\n\n"
+        text = tr("You are about to upload %1 screenshots to %2.\n"
+                  "You should double-check for personal information.\n\n"
                   "Are you sure?")
-                   .arg(selection.size());
+                   .arg(QString::number(selection.size()), baseUrl.host());
     else
-        text =
-            tr("You are about to upload the selected screenshot.\n\n"
-               "Are you sure?");
+        text = tr("You are about to upload the selected screenshot to %1.\n"
+                  "You should double-check for personal information.\n\n"
+                  "Are you sure?")
+                   .arg(baseUrl.host());
 
     auto response = CustomMessageBox::selectable(this, "Confirm Upload", text, QMessageBox::Warning, QMessageBox::Yes | QMessageBox::No,
                                                  QMessageBox::No)
@@ -400,41 +387,44 @@ void ScreenshotsPage::on_actionUpload_triggered()
 
     QList<ScreenShot::Ptr> uploaded;
     auto job = NetJob::Ptr(new NetJob("Screenshot Upload", APPLICATION->network()));
-    if(selection.size() < 2)
-    {
+
+    ProgressDialog dialog(this);
+    dialog.setSkipButton(true, tr("Abort"));
+
+    if (selection.size() < 2) {
         auto item = selection.at(0);
         auto info = m_model->fileInfo(item);
         auto screenshot = std::make_shared<ScreenShot>(info);
         job->addNetAction(ImgurUpload::make(screenshot));
 
-        m_uploadActive = true;
-        ProgressDialog dialog(this);
+        connect(job.get(), &Task::failed, [this](QString reason) {
+            CustomMessageBox::selectable(this, tr("Failed to upload screenshots!"), reason, QMessageBox::Critical)->show();
+        });
+        connect(job.get(), &Task::aborted, [this] {
+            CustomMessageBox::selectable(this, tr("Screenshots upload aborted"), tr("The task has been aborted by the user."),
+                                         QMessageBox::Information)
+                ->show();
+        });
 
-        if(dialog.execWithTask(job.get()) != QDialog::Accepted)
-        {
-            CustomMessageBox::selectable(this, tr("Failed to upload screenshots!"),
-                                         tr("Unknown error"), QMessageBox::Warning)->exec();
-        }
-        else
-        {
+        m_uploadActive = true;
+
+        if (dialog.execWithTask(job.get()) == QDialog::Accepted) {
             auto link = screenshot->m_url;
-            QClipboard *clipboard = QApplication::clipboard();
+            QClipboard* clipboard = QApplication::clipboard();
+            qDebug() << "ImgurUpload link" << link;
             clipboard->setText(link);
             CustomMessageBox::selectable(
-                    this,
-                    tr("Upload finished"),
-                    tr("The <a href=\"%1\">link  to the uploaded screenshot</a> has been placed in your clipboard.")
-                        .arg(link),
-                    QMessageBox::Information
-            )->exec();
+                this, tr("Upload finished"),
+                tr("The <a href=\"%1\">link  to the uploaded screenshot</a> has been placed in your clipboard.").arg(link),
+                QMessageBox::Information)
+                ->exec();
         }
 
         m_uploadActive = false;
         return;
     }
 
-    for (auto item : selection)
-    {
+    for (auto item : selection) {
         auto info = m_model->fileInfo(item);
         auto screenshot = std::make_shared<ScreenShot>(info);
         uploaded.push_back(screenshot);
@@ -442,32 +432,36 @@ void ScreenshotsPage::on_actionUpload_triggered()
     }
     SequentialTask task;
     auto albumTask = NetJob::Ptr(new NetJob("Imgur Album Creation", APPLICATION->network()));
-    auto imgurAlbum = ImgurAlbumCreation::make(uploaded);
+    auto imgurResult = std::make_shared<ImgurAlbumCreation::Result>();
+    auto imgurAlbum = ImgurAlbumCreation::make(imgurResult, uploaded);
     albumTask->addNetAction(imgurAlbum);
     task.addTask(job);
     task.addTask(albumTask);
+
+    connect(&task, &Task::failed, [this](QString reason) {
+        CustomMessageBox::selectable(this, tr("Failed to upload screenshots!"), reason, QMessageBox::Critical)->show();
+    });
+    connect(&task, &Task::aborted, [this] {
+        CustomMessageBox::selectable(this, tr("Screenshots upload aborted"), tr("The task has been aborted by the user."),
+                                     QMessageBox::Information)
+            ->show();
+    });
+
     m_uploadActive = true;
-    ProgressDialog prog(this);
-    if (prog.execWithTask(&task) != QDialog::Accepted)
-    {
-        CustomMessageBox::selectable(
-            this,
-            tr("Failed to upload screenshots!"),
-            tr("Unknown error"),
-            QMessageBox::Warning
-        )->exec();
-    }
-    else
-    {
-        auto link = QString("https://imgur.com/a/%1").arg(imgurAlbum->id());
-        QClipboard *clipboard = QApplication::clipboard();
-        clipboard->setText(link);
-        CustomMessageBox::selectable(
-            this,
-            tr("Upload finished"),
-            tr("The <a href=\"%1\">link  to the uploaded album</a> has been placed in your clipboard.") .arg(link),
-            QMessageBox::Information
-        )->exec();
+    if (dialog.execWithTask(&task) == QDialog::Accepted) {
+        if (imgurResult->id.isEmpty()) {
+            CustomMessageBox::selectable(this, tr("Failed to upload screenshots!"), tr("Unknown error"), QMessageBox::Warning)->exec();
+        } else {
+            auto link = QString("https://imgur.com/a/%1").arg(imgurResult->id);
+            qDebug() << "ImgurUpload link" << link;
+            QClipboard* clipboard = QApplication::clipboard();
+            clipboard->setText(link);
+            CustomMessageBox::selectable(
+                this, tr("Upload finished"),
+                tr("The <a href=\"%1\">link  to the uploaded album</a> has been placed in your clipboard.").arg(link),
+                QMessageBox::Information)
+                ->exec();
+        }
     }
     m_uploadActive = false;
 }
@@ -475,8 +469,7 @@ void ScreenshotsPage::on_actionUpload_triggered()
 void ScreenshotsPage::on_actionCopy_Image_triggered()
 {
     auto selection = ui->listView->selectionModel()->selectedRows();
-    if(selection.size() < 1)
-    {
+    if (selection.size() < 1) {
         return;
     }
 
@@ -491,15 +484,13 @@ void ScreenshotsPage::on_actionCopy_Image_triggered()
 void ScreenshotsPage::on_actionCopy_File_s_triggered()
 {
     auto selection = ui->listView->selectionModel()->selectedRows();
-    if(selection.size() < 1)
-    {
+    if (selection.size() < 1) {
         // Don't do anything so we don't empty the users clipboard
         return;
     }
 
     QString buf = "";
-    for (auto item : selection)
-    {
+    for (auto item : selection) {
         auto info = m_model->fileInfo(item);
         buf += "file:///" + info.absoluteFilePath() + "\r\n";
     }
@@ -531,8 +522,7 @@ void ScreenshotsPage::on_actionDelete_triggered()
     if (response != QMessageBox::Yes)
         return;
 
-    for (auto item : selected)
-    {
+    for (auto item : selected) {
         if (FS::trash(m_model->filePath(item)))
             continue;
 
@@ -551,23 +541,19 @@ void ScreenshotsPage::on_actionRename_triggered()
 
 void ScreenshotsPage::openedImpl()
 {
-    if(!m_valid)
-    {
+    if (!m_valid) {
         m_valid = FS::ensureFolderPathExists(m_folder);
     }
-    if (m_valid)
-    {
+    if (m_valid) {
         QString path = QDir(m_folder).absolutePath();
         auto idx = m_model->setRootPath(path);
-        if(idx.isValid())
-        {
+        if (idx.isValid()) {
             ui->listView->setModel(m_filterModel.get());
-            connect(ui->listView->selectionModel(), &QItemSelectionModel::selectionChanged, this, &ScreenshotsPage::onCurrentSelectionChanged);
-            onCurrentSelectionChanged(ui->listView->selectionModel()->selection()); // set initial button enable states
+            connect(ui->listView->selectionModel(), &QItemSelectionModel::selectionChanged, this,
+                    &ScreenshotsPage::onCurrentSelectionChanged);
+            onCurrentSelectionChanged(ui->listView->selectionModel()->selection());  // set initial button enable states
             ui->listView->setRootIndex(m_filterModel->mapFromSource(idx));
-        }
-        else
-        {
+        } else {
             ui->listView->setModel(nullptr);
         }
     }
